<?php
/**
 * Project:
 * CONTENIDO Content Management System
 *
 * Description:
 * CONTENIDO module input class for Slideshow
 *
 * Requirements:
 * @con_php_req 5.0
 *
 * @package     CONTENIDO_Modules
 * @subpackage  Slideshow
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2011 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id: $
 */


if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

/**
 * CONTENIDO module input class for Slideshow
 */
class ModuleSlideshowInput
{

    /**
     * Default slideshow speed
     * @var  int
     */
    const DEFAULT_SPEED = 4000;

    /**
     * Default effect duration
     * @var  float
     */
    const DEFAULT_DURATION = 1.0;

    /**
     * Assoziative order array
     * @var  array
     */
    protected $_aOrder;

    /**
     * Client id
     * @var  int
     */
    protected $_client;

    /**
     * Module translations
     * @var  array
     */
    protected $_i18n = array();

    /**
     * Selected slideshow order
     * @var  string
     */
    public $selectedOrder;

    /**
     * Selected folder containing images for the slideshow
     * @var  string
     */
    public $selectedDirname;

    /**
     * Use subdirectories in selected folder
     * @var  string
     */
    public $useSubdirectoriesChk;

    /**
     * Slideshow identifier
     * @var  string
     */
    public $elementId;

    /**
     * Speed in milliseconds (time between two slides)
     * @var  int
     */
    public $speed;

    /**
     * Duration of the effect
     * @var  float
     */
    public $duration;

    /**
     * Horizontal align
     * @var  string
     */
    public $centerHorizontalChk;

    /**
     * Max number of images to display
     * @var  int
     */
    public $maxImages;

    /**
     * Max width of slideshow images
     * @var  int
     */
    public $maxWidth;

    /**
     * Max height of slideshow images
     * @var  int
     */
    public $maxHeight;

    /**
     * Max cache time of resized images in minutes
     * @var  int
     */
    public $maxCachetime;


    /**
     * Constructor sets some properties
     *
     * @param  int    $clientId   Client id
     * @param  array  $aConfig  Module configuration
     * @param  array  $aTranslations  Assoziative translations list
     */
    public function __construct($clientId, array $aConfig, array $aTranslations)
    {
        $this->_client              = (int) $clientId;
        $this->_i18n                = $aTranslations;
        $this->elementId            = trim($aConfig['elementId']);
        $this->selectedDirname      = $aConfig['selectedDirname'];
        $this->useSubdirectoriesChk = ($aConfig['useSubdirectoriesChk'] != '') ? ' checked="checked"' : '';
        $this->speed                = (int) $aConfig['speed'];
        $this->duration             = (float) $aConfig['duration'];
        $this->centerHorizontalChk  = ($aConfig['centerHorizontalChk'] != '') ? ' checked="checked"' : '';
        $this->maxImages            = (int) $aConfig['maxImages'];
        $this->maxWidth             = (int) $aConfig['maxWidth'];
        $this->maxHeight            = (int) $aConfig['maxHeight'];
        $this->maxCachetime         = (int) $aConfig['maxCachetime'];
        $this->selectedOrder        = $aConfig['selectedOrder'];
        $this->_aOrder = array(
            'RAND()'         => $this->_i18n['random'],
            'filename:ASC'   => $this->_i18n['filename_asc'],
            'filename:DESC'  => $this->_i18n['filename_desc'],
            'size:ASC'       => $this->_i18n['size_asc'],
            'size:DESC'      => $this->_i18n['filename_desc'],
            'filetype:ASC'   => $this->_i18n['filetype_asc'],
            'filetype:DESC'  => $this->_i18n['filetype_desc'],
            'created:ASC'    => $this->_i18n['created_asc'],
            'created:esc'    => $this->_i18n['created_esc'],
            'idupl:ASC'      => $this->_i18n['id_asc'],
            'idupl:DESC'     => $this->_i18n['id_desc']
        );

        if ($this->elementId == '') {
            $this->elementId = 'slideshow';
        }

        if ($this->maxImages <= 1) {
            $this->maxImages = '';
        }

        if ($this->speed <= 0) {
            $this->speed = self::DEFAULT_SPEED;
        }

        if ($this->duration <= 0) {
            $this->duration = self::DEFAULT_DURATION;
        }

        if ($this->maxWidth <= 0) {
            $this->maxWidth = '';
        }

        if ($this->maxHeight <= 0) {
            $this->maxHeight = '';
        }

        if ($this->maxCachetime <= 0) {
            $this->maxCachetime = '';
        }
    }


    /**
     * Sets module translations
     * @param  array  $translations  Assoziative translations list
     */
    public function setMi18n(array $aTranslations)
    {
        $this->_i18n = array_merge($this->_i18n, $translations);
    }


    /**
     * Generates and returns option items of the dirname select box.
     *
     * @return  string  Composed option items.
     */
    public function generateDirSelectOptions()
    {
        $oUploadColl = new UploadCollection();
        $oUploadColl->flexSelect('dirname', '', 'idclient=' . $this->_client, 'dirname');

        $opt = '<option value="">' . $this->_i18n['__select_folder__'] . '</option>' . "\n";
        while ($oUploadItem = $oUploadColl->next()) {
            $dirname = $oUploadItem->get('dirname');
            $sel = ($dirname == $this->selectedDirname) ? ' selected="selected"' : '';
            $opt .= '<option value="' . $dirname . '"' . $sel . '>' . $dirname . '</option>' . "\n";
        }
        return $opt;
    }


    /**
     * Generates and returns option items of the order select box.
     *
     * @return  string  Composed option items.
     */
    public function generateOrderSelectOptions()
    {
        $opt = '<option value="">' . $this->_i18n['__select_order__'] . '</option>' . "\n";
        foreach ($this->_aOrder as $key => $value) {
            $sel = ($key == $this->selectedOrder) ? ' selected="selected"' : '';
            $opt .= '<option value="' . $key . '"' . $sel . '>' . $value . '</option>' . "\n";
        }
        return $opt;
    }

}
