<?php
/**
 * Project:
 * CONTENIDO Content Management System
 *
 * Description:
 * CONTENIDO module output class for Slideshow
 *
 * Requirements:
 * @con_php_req 5.0
 *
 * @package     CONTENIDO_Modules
 * @subpackage  Slideshow
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2011 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id: $
 */

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

/**
 * CONTENIDO module output class for Slideshow
 */
class ModuleSlideshowOutput
{
    /**
     * Default slideshow speed
     * @var  int
     */
    const DEFAULT_SPEED = 4000;

    /**
     * Default effect duration
     * @var  float
     */
    const DEFAULT_DURATION = 1.0;

    /**
     * Default cache time of resized images in minutes
     * @var  int
     */
    const DEFAULT_CACHETIME = 10;

    /**
     * Default image width
     * @var  string
     */
    const DEFAULT_WIDTH = '100%';

    /**
     * Default image height
     * @var  string
     */
    const DEFAULT_HEIGHT = '100%';

    /**
     * Supported filetypes for the slideshow
     * @var  string
     */
    const FILE_TYPES = "'jpg','jpeg','png','gif'";

    /**
     * Module translations
     * @var  array
     */
    protected $_i18n = array();

    /**
     * To store occured errors
     * @var  string
     */
    protected $_sError = '';

    /**
     * Max calculated image width
     * @var  int
     */
    protected $_iCalculatedMaxWidth = 0;

    /**
     * Max calculated image height
     * @var  int
     */
    protected $_iCalculatedMaxHeight = 0;

    /**
     * Language id
     * @var  int
     */
    protected $_lang;

    /**
     * Assoziative order array
     * @var  array
     */
    protected $_aOrder;

    /**
     * Client HTML path
     * @var  string
     */
    protected $_sHtmlPath;

    /**
     * Client upload directory
     * @var  string
     */
    protected $_sUploadDir;

    /**
     * Flag to use subdirectories
     * @var  string
     */
    protected $_bUseSubdirs;

    /**
     * Absolute path to client upload directory
     * @var  string
     */
    protected $_sAbsUploadPath;

    /**
     * Flag to center images horizontaly
     * @var  bool
     */
    protected $_bCenterImagesHorizontal = false;

    /**
     * Slideshow id
     * @var  string
     */
    public $elementId;

    /**
     * Selected slideshow directory
     * @var  string
     */
    public $selectedDirname;

    /**
     * Slideshow speed in miliseconds
     * @var  int
     */
    public $speed;

    /**
     * Duration of the effect
     * @var  float
     */
    public $duration;

    /**
     * Max number of images to display
     * @var  int
     */
    public $maxImages;

    /**
     * Max allowed image width
     * @var  int
     */
    public $maxWidth;

    /**
     * Max allowed image height
     * @var  int
     */
    public $maxHeight;

    /**
     * Max cache time for resized images in minutes
     * @var  int
     */
    public $maxCachetime;

    /**
     * Slideshow order
     * @var  string
     */
    public $selectedOrder;


    /**
     * Constructor sets some properties...
     *
     * @param  array  $aClientCfg  Client configuration
     * @param  int    $iLangId     Language id
     * @param  array  $aConfig   Module configuration
     * @param  array  $aTranslations  Assoziative translations list
     */
    public function __construct($aClientCfg, $iLangId, array $aConfig, array $aTranslations)
    {
        $this->_lang           = (int) $iLangId;
        $this->_i18n           = $aTranslations;
        $this->_sHtmlPath      = $aClientCfg['path']['htmlpath'];
        $this->_sUploadDir     = $aClientCfg['upl']['frontendpath'];
        $this->_bUseSubdirs    = ($aConfig['useSubdirectoriesChk'] != '') ? true : false;
        $this->_sAbsUploadPath = $aClientCfg['upl']['path'];

        $this->_aOrder = array(
            'RAND()'         => $this->_i18n['random'],
            'filename:ASC'   => $this->_i18n['filename_asc'],
            'filename:DESC'  => $this->_i18n['filename_desc'],
            'size:ASC'       => $this->_i18n['size_asc'],
            'size:DESC'      => $this->_i18n['filename_desc'],
            'filetype:ASC'   => $this->_i18n['filetype_asc'],
            'filetype:DESC'  => $this->_i18n['filetype_desc'],
            'created:ASC'    => $this->_i18n['created_asc'],
            'created:esc'    => $this->_i18n['created_esc'],
            'idupl:ASC'      => $this->_i18n['id_asc'],
            'idupl:DESC'     => $this->_i18n['id_desc']
        );

        // slideshow id
        $this->elementId = trim($aConfig['elementId']);
        if ($this->elementId == '') {
            $this->elementId = 'slideshow';
        }

        // directory including images 4 the slideshow
        $this->selectedDirname = trim($aConfig['selectedDirname']);
        if ($this->selectedDirname == '') {
            $this->_sError = "Slideshow: No image folder selected!";
            return;
        } elseif (!is_dir($this->_sUploadDir . $this->selectedDirname)) {
            $this->_sError = "Slideshow: Selected image folder doesn't exists anymore!";
            return;
        }

        // slideshowspeed
        $this->speed = (int) $aConfig['speed'];
        if ($this->speed <= 0) {
            $this->speed = self::DEFAULT_SPEED;
        }

        // effect duration
        $this->duration = (float) $aConfig['duration'];
        if ($this->duration <= 0) {
            $this->duration = self::DEFAULT_DURATION;
        }

        // flag 2 center all images horizontally
        $this->_bCenterImagesHorizontal = ($aConfig['centerHorizontalChk'] != '') ? true : false;

        // number of max images to display
        $this->maxImages = (int) $aConfig['maxImages'];
        if ($this->maxImages <= 1) {
            $this->maxImages = '';
        }
        
        // max allowed width of images. bigger ones will be resized
        $this->maxWidth = (int) $aConfig['maxWidth'];
        if ($this->maxWidth <= 0) {
            $this->maxWidth = '';
        }

        // max allowed height of images. bigger ones will also be resized
        $this->maxHeight = (int) $aConfig['maxHeight'];
        if ( $this->maxHeight <= 0) {
            $this->maxHeight = '';
        }

        // max cachetime in minutes 4 resized images
        $this->maxCachetime = (int) $aConfig['maxCachetime'];
        if ($this->maxCachetime <= 0) {
            $this->maxCachetime = self::DEFAULT_CACHETIME;
        } elseif ($this->maxWidth == '' && $this->maxHeight == '') {
            $this->maxCachetime = self::DEFAULT_CACHETIME;
        }

        // selected order type
        $this->selectedOrder = $aConfig['selectedOrder'];
        if (!isset($this->selectedOrder[$this->_aOrder])) {
            $this->selectedOrder = 'filename:ASC';
        }
    }


    /**
     * Generates the slideshow or a message on any occured error.
     */
    public function generateOutput()
    {
        if ($this->_sError !== '') {
            print $this->_sError;
            return;
        }

        $oTemplate = new Template();

        // get images
        $aImages = $this->_getImages();
        if (count($aImages) == 0) {
            print "Slideshow: No images found in defined image folder!";
            return;
        }

        // max width and height of slideshow box
        $sStyleDimensions  = (is_numeric($this->_iCalculatedMaxWidth)) ? 'width:' . $this->_iCalculatedMaxWidth . 'px;' : 'width:' . self::DEFAULT_WIDTH . ';';
        $sStyleDimensions .= (is_numeric($this->_iCalculatedMaxHeight)) ? 'height:' . $this->_iCalculatedMaxHeight . 'px;' : 'height:' . self::DEFAULT_HEIGHT . ';';
        $sStyleDimensions  = sprintf(' style="%s"', $sStyleDimensions);

        $oTemplate->set('s', 'SLIDESHOW.ID', $this->elementId);
        $oTemplate->set('s', 'SLIDESHOW.ATTRIBUTES', $sStyleDimensions);

        // loop images array an fill template
        foreach ($aImages as $image) {
            if ($this->_bCenterImagesHorizontal == true) {
                // comopse css definition 2 center a image horizontally
                $sCenterCss = $this->_css2centerImageHorizontal($image['size']);
                if ($sCenterCss != '') {
                    $image['attr'] .= ' style="' . $sCenterCss . '"';
                }
            }

            $oTemplate->set('d', 'SLIDE.ATTRIBUTES', $sStyleDimensions);
            $oTemplate->set('d', 'IMG.SRC', $image['src']);
            $oTemplate->set('d', 'IMG.ALT', $image['alt']);
            $oTemplate->set('d', 'IMG.ATTRIBUTES', $image['attr']);
            $oTemplate->next();
        }

        $oTemplate->set('s', 'SLIDESHOW.SPEED', $this->speed);
        $oTemplate->set('s', 'SLIDESHOW.DURATION', $this->duration);
        $oTemplate->set('s', 'NOSCRIPT', $this->_i18n['noscript_msg']);

        $oTemplate->generate('templates/slideshow.html', 0, 0);
    }


    /**
     * Builds the images query statement, executes it and returns found images.
     *
     * @return array List of found images
     */
    protected function _getImages()
    {
        // where statement with selected dir and supported filetypes
        $aWhere = array();
        if ($this->_bUseSubdirs) {
            $aWhere[] = 'dirname LIKE "' . $this->selectedDirname . '%"';
        } else {
            $aWhere[] = 'dirname="' . $this->selectedDirname . '"';
        }
        $aWhere[] = 'AND';
        $aWhere[] = 'LOWER(filetype) IN(' . self::FILE_TYPES . ')';
        $sWhere = implode(' ', $aWhere);

        // order settings
        if (strpos($this->selectedOrder, ':') > 0) {
            list($sort, $sortdir) = explode(':', $this->selectedOrder);
            $sOrder = $sort . ' ' . $sortdir;
        } else {
            $sOrder = $this->selectedOrder;
        }

        // limit
        if ((int) $this->maxImages > 0) {
            $sLimit = '0, ' . $this->maxImages;
        } else {
            $sLimit = '';
        }

        // run the statement
        $oUploadColl = new UploadCollection();
        $oUploadColl->select($sWhere, '', $sOrder, $sLimit);

        $aImages = array();

        // iterate thru upload collection 2 store data in array
        while ($oUploadItem = $oUploadColl->next()) {

            // some checks
            $sImageFile = $this->_sAbsUploadPath . $oUploadItem->get('dirname') . $oUploadItem->get('filename');
            if (!is_file($sImageFile) || !is_readable($sImageFile)) {
                continue;
            }

            if (is_numeric($this->maxWidth) && is_numeric($this->maxHeight)) {
                // bigger images have 2 be resized
                $sImageFile = capiImgScale(
                    $sImageFile, $this->maxWidth, $this->maxHeight, false, false, $this->maxCachetime
                );
                if (!$sImageFile) {
                    continue;
                }
                $sImageFile = str_replace($this->_sHtmlPath, '', $sImageFile);
            } else {
                // use original image file
                $sImageFile = $this->_sUploadDir . $oUploadItem->get('dirname') . $oUploadItem->get('filename');
            }

            // get'n store image dimensions
            $size = $this->_getImageSize($sImageFile);
            $attr = (is_array($size)) ? ' ' . $size[3] : '';

            // add new images array item
            $aImages[$oUploadItem->get('idupl')] = array(
                'size'  => $size,
                'src'   => $sImageFile,
                'alt'   => '',
                'attr'  => $attr
            );
        }

        if (count($aImages) > 0) {
            // now get description by language
            $sWhere = '';
            foreach ($aImages as $idUpl => $item) {
                $sWhere .= ' idupl=' . $idUpl . ' OR ';
            }
            $sWhere = substr($sWhere, 0, -3) . ' AND idlang=' . $this->_lang;
            $oUploadMetaColl = new UploadMetaCollection();
            $oUploadMetaColl->select($sWhere, '', '');

            // iterate upload meta collection 2 store description in images array
            while ($oItem = $oUploadMetaColl->next()) {
                $aImages[$oItem->get('idupl')]['alt'] = htmlspecialchars($oItem->get('description'));
            }
        }

        return $aImages;
    }


    /**
     * Returns value of getimagesize function, and also stores maximum width/height of
     * existing images.
     *
     * @param string Image file to ger size array for
     * @return array Returnvalue of getimagesize() function
     */
    protected function _getImageSize($file)
    {
        $size = @getimagesize($file);
        if (is_array($size)) {
            if ($this->_iCalculatedMaxWidth < $size[0]) {
                $this->_iCalculatedMaxWidth = $size[0];
            }
            if ($this->_iCalculatedMaxHeight < $size[1]) {
                $this->_iCalculatedMaxHeight = $size[1];
            }
        }
        return $size;
    }


    /**
     * Composes css definition 2 center a image horizontally and returns it back.
     *
     * @param mixed Array including image size imnformations (result of getimagesize())
     * @return string Composed css definition
     */
    protected function _css2centerImageHorizontal($size)
    {
        if (!is_array($size) || $this->_iCalculatedMaxHeight == 0 || ($size[1] == $this->_iCalculatedMaxHeight)) {
            return '';
        }
        $css = 'margin-top:' . (($this->_iCalculatedMaxHeight - $size[1]) / 2) . 'px';
        return $css;
    }

}
